<?php

namespace IQBall\App;

use IQBall\Core\Http\HttpCodes;
use IQBall\Core\Http\HttpRequest;
use IQBall\Core\Http\HttpResponse;
use IQBall\Core\Validation\ValidationFail;
use IQBall\Core\Validation\Validator;

class Control {
    /**
     * Runs given callback, if the request's json validates the given schema.
     * @param array<string, Validator[]> $schema an array of `fieldName => Validators` which represents the request object schema
     * @param callable(HttpRequest): HttpResponse $run the callback to run if the request is valid according to the given schema.
     *                      THe callback must accept an HttpRequest, and return an HttpResponse object.
     * @return HttpResponse
     */
    public static function runChecked(array $schema, callable $run): HttpResponse {
        $request_body = file_get_contents('php://input');
        $payload_obj = json_decode($request_body);
        if (!$payload_obj instanceof \stdClass) {
            $fail = new ValidationFail("bad-payload", "request body is not a valid json object");
            return ViewHttpResponse::twig("error.html.twig", ["failures" => [$fail]], HttpCodes::BAD_REQUEST);
        }
        $payload = get_object_vars($payload_obj);
        return self::runCheckedFrom($payload, $schema, $run);
    }

    /**
     * Runs given callback, if the given request data array validates the given schema.
     * @param array<string, mixed> $data the request's data array.
     * @param array<string, Validator[]> $schema an array of `fieldName => Validators` which represents the request object schema
     * @param callable(HttpRequest): HttpResponse $run the callback to run if the request is valid according to the given schema.
     *                      THe callback must accept an HttpRequest, and return an HttpResponse object.
     * @return HttpResponse
     */
    public static function runCheckedFrom(array $data, array $schema, callable $run): HttpResponse {
        $fails = [];
        $request = HttpRequest::from($data, $fails, $schema);

        if (!empty($fails)) {
            return ViewHttpResponse::twig("error.html.twig", ['failures' => $fails], HttpCodes::BAD_REQUEST);
        }

        return call_user_func_array($run, [$request]);
    }
}
