<?php

namespace IQBall\App\Controller;

use IQBall\App\Session\SessionHandle;
use IQBall\App\ViewHttpResponse;
use IQBall\Core\Http\HttpRequest;
use IQBall\Core\Http\HttpResponse;
use IQBall\Core\Model\TeamModel;
use IQBall\Core\Validation\FieldValidationFail;
use IQBall\Core\Validation\Validators;

class TeamController {
    private TeamModel $model;

    /**
     * @param TeamModel $model
     */
    public function __construct(TeamModel $model) {
        $this->model = $model;
    }

    /**
     * @param SessionHandle $session
     * @return ViewHttpResponse the team creation panel
     */
    public function displayCreateTeam(SessionHandle $session): ViewHttpResponse {
        return ViewHttpResponse::twig("insert_team.html.twig", []);
    }


    /**
     * @param SessionHandle $session
     * @return ViewHttpResponse the team panel to add a member
     */
    public function displayAddMember(SessionHandle $session): ViewHttpResponse {
        return ViewHttpResponse::twig("add_member.html.twig", []);
    }


    /**
     * @param SessionHandle $session
     * @return ViewHttpResponse the team panel to delete a member
     */
    public function displayDeleteMember(SessionHandle $session): ViewHttpResponse {
        return ViewHttpResponse::twig("delete_member.html.twig", []);
    }

    /**
     * create a new team from given request name, mainColor, secondColor and picture url
     * @param array<string, mixed> $request
     * @param SessionHandle $session
     * @return HttpResponse
     */
    public function submitTeam(array $request, SessionHandle $session): HttpResponse {
        $failures = [];
        $request = HttpRequest::from($request, $failures, [
            "name" => [Validators::lenBetween(1, 32), Validators::nameWithSpaces()],
            "main_color" => [Validators::hexColor()],
            "second_color" => [Validators::hexColor()],
            "picture" => [Validators::isURL()],
        ]);
        if (!empty($failures)) {
            $badFields = [];
            foreach ($failures as $e) {
                if ($e instanceof FieldValidationFail) {
                    $badFields[] = $e->getFieldName();
                }
            }
            return ViewHttpResponse::twig('insert_team.html.twig', ['bad_fields' => $badFields]);
        }
        $teamId = $this->model->createTeam($request['name'], $request['picture'], $request['main_color'], $request['second_color']);
        return $this->displayTeam($teamId, $session);
    }

    /**
     * @param SessionHandle $session
     * @return ViewHttpResponse the panel to search a team by its name
     */
    public function displayListTeamByName(SessionHandle $session): ViewHttpResponse {
        return ViewHttpResponse::twig("list_team_by_name.html.twig", []);
    }

    /**
     * returns a view that contains all the teams description whose name matches the given name needle.
     * @param array<string, mixed> $request
     * @param SessionHandle $session
     * @return HttpResponse
     */
    public function listTeamByName(array $request, SessionHandle $session): HttpResponse {
        $errors = [];
        $request = HttpRequest::from($request, $errors, [
            "name" => [Validators::lenBetween(1, 32), Validators::nameWithSpaces()],
        ]);

        if (!empty($errors) && $errors[0] instanceof FieldValidationFail) {
            $badField = $errors[0]->getFieldName();
            return ViewHttpResponse::twig('list_team_by_name.html.twig', ['bad_field' => $badField]);
        }

        $teams = $this->model->listByName($request['name']);

        if (empty($teams)) {
            return ViewHttpResponse::twig('display_teams.html.twig', []);
        }

        return ViewHttpResponse::twig('display_teams.html.twig', ['teams' => $teams]);
    }

    /**
     * @param int $id
     * @param SessionHandle $session
     * @return ViewHttpResponse a view that displays given team information
     */
    public function displayTeam(int $id, SessionHandle $session): ViewHttpResponse {
        $result = $this->model->getTeam($id);
        return ViewHttpResponse::twig('display_team.html.twig', ['team' => $result]);
    }

    /**
     * add a member to a team
     * @param array<string, mixed> $request
     * @param SessionHandle $session
     * @return HttpResponse
     */
    public function addMember(array $request, SessionHandle $session): HttpResponse {
        $errors = [];

        $request = HttpRequest::from($request, $errors, [
            "team" => [Validators::isInteger()],
            "email" => [Validators::email(), Validators::lenBetween(5, 256)],
        ]);

        $teamId = intval($request['team']);
        $this->model->addMember($request['email'], $teamId, $request['role']);
        return $this->displayTeam($teamId, $session);
    }

    /**
     * remove a member from a team
     * @param array<string, mixed> $request
     * @param SessionHandle $session
     * @return HttpResponse
     */
    public function deleteMember(array $request, SessionHandle $session): HttpResponse {
        $errors = [];

        $request = HttpRequest::from($request, $errors, [
            "team" => [Validators::isInteger()],
            "email" => [Validators::email(), Validators::lenBetween(5, 256)],
        ]);

        return $this->displayTeam($this->model->deleteMember($request['email'], intval($request['team'])), $session);
    }
}
